﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectDumpTruck;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
internal class DrawningDumpTruck
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityDumpTruck? EntityDumpTruck { get; private set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    private int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    private int? _startPosY;
    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    private readonly int _drawningTruckWidth = 135;
    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    private readonly int _drawningTruckHeight = 85;
    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="awning">Признак наличия обвеса</param>
    /// <param name="tent">Признак наличия антикрыла</param>
    public void Init(int speed, double weight, Color bodyColor, Color
    additionalColor, bool awning, bool tent)
    {
        EntityDumpTruck = new EntityDumpTruck();
        EntityDumpTruck.Init(speed, weight, bodyColor, additionalColor, awning, tent);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена
    public bool SetPictureSize(int width, int height)

    {

        if (width >= _drawningTruckWidth && height >= _drawningTruckHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX != null && _startPosY != null)
            {
                if (_startPosX.Value + _drawningTruckWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningTruckWidth;
                }
                if (_startPosY.Value + _drawningTruckHeight > _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawningTruckHeight;
                }
            }
            return true;
        }
        return false;

    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)

    {

        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        else
        {
            _startPosX = x;
            _startPosY = y;

            if (_startPosX < 0) _startPosX = 0;
            if (_startPosY < 0) _startPosY = 0;

            if (_startPosX + _drawningTruckWidth > _pictureWidth.Value)
            {
                _startPosX = _pictureWidth.Value - _drawningTruckWidth;
            }
            if (_startPosY + _drawningTruckHeight > _pictureHeight.Value)
            {
                _startPosY = _pictureHeight.Value - _drawningTruckHeight;
            }
        }
    }

    public bool MoveTransport(DirectionType direction)
    {
        if (EntityDumpTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityDumpTruck.Step > 0)
                {
                    _startPosX -= (int)EntityDumpTruck.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityDumpTruck.Step > 0)
                {
                    _startPosY -= (int)EntityDumpTruck.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityDumpTruck.Step + _drawningTruckHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityDumpTruck.Step;
                }
                return true;
            //вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityDumpTruck.Step + _drawningTruckWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityDumpTruck.Step;
                }
                return true;
            default:
                return false;
        }
    }

    public void DrawTransport(Graphics g)
    {
        if (EntityDumpTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityDumpTruck.AdditionalColor);

        //Кабина и основа
        Brush body = new SolidBrush(EntityDumpTruck.BodyColor);
        g.FillRectangle(body, _startPosX.Value + 100, _startPosY.Value, 30, 40);
        g.DrawRectangle(pen, _startPosX.Value + 100, _startPosY.Value, 30, 40);

        g.FillRectangle(body, _startPosX.Value, _startPosY.Value + 40, 130, 20);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 40, 130, 20);

        //Фары
        Brush light = new SolidBrush(Color.Yellow);
        g.FillRectangle(light, _startPosX.Value + 130, _startPosY.Value + 40, 4, 5);
        g.FillRectangle(light, _startPosX.Value - 5, _startPosY.Value + 40, 4, 5);

        Brush light1 = new SolidBrush(Color.DarkOrange);
        g.FillRectangle(light1, _startPosX.Value + 125, _startPosY.Value + 40, 5, 5);
        g.FillRectangle(light1, _startPosX.Value, _startPosY.Value + 40, 5, 5);

        //Стекло
        Brush glass = new SolidBrush(Color.LightBlue);
        g.FillRectangle(glass, _startPosX.Value + 110, _startPosY.Value + 5, 20, 25);

        //Отрисовка колёс
        Brush wheels = new SolidBrush(Color.Black);
        g.FillEllipse(wheels, _startPosX.Value, _startPosY.Value + 60, 30, 30);
        g.FillEllipse(wheels, _startPosX.Value + 30, _startPosY.Value + 60, 30, 30);
        g.FillEllipse(wheels, _startPosX.Value + 100, _startPosY.Value + 60, 30, 30);

        Brush kolpac = new SolidBrush(Color.Gray);
        g.FillEllipse(kolpac, _startPosX.Value + 6, _startPosY.Value + 66, 17, 17);
        g.FillEllipse(kolpac, _startPosX.Value + 36, _startPosY.Value + 66, 17, 17);
        g.FillEllipse(kolpac, _startPosX.Value + 106, _startPosY.Value + 66, 17, 17);

        //Отрисовка границ


        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 40, 130, 20);
        g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 60, 30, 30);
        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 60, 30, 30);
        g.DrawEllipse(pen, _startPosX.Value + 100, _startPosY.Value + 60, 30, 30);

        Pen glass_ = new(Color.DarkGray);
        g.DrawRectangle(glass_, _startPosX.Value + 110, _startPosY.Value + 5, 20, 25);

        Pen light_ = new(Color.LightBlue);
        g.DrawRectangle(light_, _startPosX.Value + 130, _startPosY.Value + 40, 4, 5);
        g.DrawRectangle(light_, _startPosX.Value + 125, _startPosY.Value + 40, 5, 5);
        g.DrawRectangle(light_, _startPosX.Value - 5, _startPosY.Value + 40, 4, 5);
        g.DrawRectangle(light_, _startPosX.Value, _startPosY.Value + 40, 5, 5);

        //Отрисовка кузова
        if (EntityDumpTruck.Awning)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value, 90, 35);
        }

        //Отрисовка тента
        if (EntityDumpTruck.Awning & EntityDumpTruck.Tent)
        {
            Brush tent = new SolidBrush(EntityDumpTruck.AdditionalColor);
            g.FillRectangle(tent, _startPosX.Value, _startPosY.Value, 95, 10);
            g.FillRectangle(tent, _startPosX.Value, _startPosY.Value, 95, 3);
            g.FillRectangle(tent, _startPosX.Value + 30, _startPosY.Value, 3, 40);
            g.FillRectangle(tent, _startPosX.Value + 70, _startPosY.Value, 3, 40);

            Pen t = new(Color.Black);
            g.DrawRectangle(t, _startPosX.Value, _startPosY.Value, 95, 10);
            g.DrawRectangle(t, _startPosX.Value + 30, _startPosY.Value, 3, 40);
            g.DrawRectangle(t, _startPosX.Value + 70, _startPosY.Value, 3, 40);
        }

    }
}


